//
//  XSPEC12  November 2003
//
//
#include "XSUser/Global/Global.h"        
#include "XSUser/Global/XSGlobal.h"        
#include "XSModel/GlobalContainer/DataContainer.h"
#include "XSUtil/Utils/XSutility.h"
#include "XSUtil/Parse/XSparse.h"
#include "XSContainer.h"        
#include "XSstreams.h"
#include "xsTypes.h"
#include "HandlerUtils.h"

int
XSGlobal::xsNotice(ClientData cdata,Tcl_Interp* tclInterp,int objc, Tcl_Obj* CONST  objv[])
{
    using namespace XSContainer;
    bool isChanged = false;
    bool isRespChanged = false;
    try
    {
	if (objc == 1)
	{
	    printDocs(static_cast<char*>(cdata),"?");
	}
	else
	{
	    string arg(Tcl_GetString(objv[1]));
	    if (arg == "all")
	    {
		// go through DataContainer datasets and set all 
		// noticedArrayelements
		IntegerArray dummy(2, -2), dummy2(2, -2);
		datasets->setChannels(true, dummy, dummy2);
		tcout << "\nAll channels noticed";  
		isChanged = true;
		IntegerArray tmp(0);
		isRespChanged = datasets->resetDetectors(tmp);                              
	    }
	    else
	    {
                IntegerArray iParams(0);
                StringArray specifiers(0);
                StringArray args(objc-1);
		for(int i=1; i < objc; ++i)
		    args[i-1] = string(Tcl_GetString(objv[i]));
                // iParams is just a dummy array in this context.
                XSparse::collectParams(args, iParams, specifiers);
		    
		static std::vector<Real> prevRanges;
                if (prevRanges.empty())
                {
                   prevRanges.resize(4);
                   prevRanges[0] = prevRanges[1] = 1;
                   prevRanges[2] = prevRanges[3] = -2;
                }
		static bool isReal = false;
		HandlerUtils::IgnoreNoticeParse(specifiers, prevRanges, isReal, isRespChanged, isChanged, true);
	    }
	}
	if (isChanged)
	{
	    datasets->Notify();
	}
	if (isRespChanged)
	{
	    tcout << "Existing dummy responses have been removed from all spectra."<<std::endl;
	    tcout << "Original responses (if any) have been restored."<<std::endl;	   
	}		
	tcout << std::endl;
	return globalData->autoSave(TCL_OK);
    }
    catch (YellowAlert&)
    {
       // Still need to check isChanged since modifications may
       // have been made to spectra prior to arg that caused error.
       if (isChanged)
       {
          datasets->Notify();
       }
       if (isRespChanged)
       {
	   tcout << "Existing dummy responses have been removed from all spectra."<<std::endl;
	   tcout << "Original responses (if any) have been restored."<<std::endl;	   
       }
       tcout << std::endl;
       return globalData->autoSave(TCL_ERROR);
    }
}
